package ru.dbotthepony.mc.otm.client.screen

import com.mojang.blaze3d.vertex.PoseStack
import net.minecraft.network.chat.Component
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.Items
import ru.dbotthepony.mc.otm.block.entity.storage.ItemMonitorPlayerSettings
import ru.dbotthepony.mc.otm.client.render.RenderHelper
import ru.dbotthepony.mc.otm.client.render.SkinGrid
import ru.dbotthepony.mc.otm.client.render.UVWindingOrder
import ru.dbotthepony.mc.otm.client.screen.panels.*
import ru.dbotthepony.mc.otm.client.screen.widget.ProgressGaugePanel
import ru.dbotthepony.mc.otm.menu.ItemMonitorMenu
import ru.dbotthepony.mc.otm.next
import ru.dbotthepony.mc.otm.prev

class ItemMonitorScreen(menu: ItemMonitorMenu, inventory: Inventory, title: Component) :
	MatteryScreen<ItemMonitorMenu>(menu, inventory, title) {

	override fun makeMainFrame(): FramePanel {
		val frame = FramePanel(this@ItemMonitorScreen, null, 0f, 0f, FRAME_WIDTH, FRAME_HEIGHT, getTitle())

		val topPanel = EditablePanel(this, frame)
		topPanel.height = ITEM_GRID_HEIGHT * 18f
		topPanel.dock = Dock.TOP

		val bottomPanel = EditablePanel(this, frame)
		bottomPanel.height = 3 * 18f
		bottomPanel.dock = Dock.TOP
		bottomPanel.setDockMargin(top = 6f)

		frame.height = topPanel.height + bottomPanel.height + frame.dockPadding.top + frame.dockPadding.bottom + 6f
		frame.width = 178f + frame.dockPadding.left + frame.dockPadding.right

		val viewScrollBar = ScrollBarPanel(this, topPanel, 28f + ITEM_GRID_WIDTH * 18f + 2f, 16f, ITEM_GRID_HEIGHT * 18f)
		viewScrollBar.setupRowMultiplier { menu.view.itemCount / ITEM_GRID_WIDTH }

		viewScrollBar.dock = Dock.RIGHT
		viewScrollBar.setDockMargin(left = 2f)

		val gridPanel = GridPanel(this, topPanel, width = ITEM_GRID_WIDTH * 18f, height = ITEM_GRID_HEIGHT * 18f, columns = ITEM_GRID_WIDTH, rows = ITEM_GRID_HEIGHT)
		gridPanel.dock = Dock.FILL

		for (i in 0 until ITEM_GRID_WIDTH * ITEM_GRID_HEIGHT) {
			object : AbstractSlotPanel(this@ItemMonitorScreen, gridPanel) {
				override fun getItemStack(): ItemStack {
					val index = i + viewScrollBar.getScroll(menu.view.itemCount / ITEM_GRID_WIDTH) * ITEM_GRID_WIDTH
					return menu.view.sortedView.getOrNull(index)?.stack ?: ItemStack.EMPTY
				}

				override fun mouseScrolledInner(mouse_x: Double, mouse_y: Double, scroll: Double): Boolean {
					return viewScrollBar.mouseScrolledInner(mouse_x, mouse_y, scroll)
				}

				override fun mouseClickedInner(mouse_x: Double, mouse_y: Double, mouse_click_type: Int): Boolean {
					val index = i + viewScrollBar.getScroll(menu.view.itemCount / ITEM_GRID_WIDTH) * ITEM_GRID_WIDTH
					menu.view.mouseClick(index, mouse_click_type)
					return true
				}
			}
		}

		val craftingGrid = GridPanel(this, bottomPanel, width = 3 * 18f, height = 3 * 18f, columns = 3, rows = 3)
		craftingGrid.dock = Dock.LEFT

		for (i in 0 until 9) {
			object : AbstractSlotPanel(this@ItemMonitorScreen, craftingGrid) {
				override fun getItemStack(): ItemStack {
					return ItemStack(Items.APPLE, i + 1)
				}
			}
		}

		val arrowAndButtons = object : EditablePanel(this@ItemMonitorScreen, bottomPanel, width = ProgressGaugePanel.GAUGE_BACKGROUND.w) {
			override fun innerRender(stack: PoseStack, mouse_x: Float, mouse_y: Float, flag: Float) {
				ProgressGaugePanel.GAUGE_BACKGROUND.render(stack, y = height / 2f - ProgressGaugePanel.GAUGE_BACKGROUND.h / 2f)
			}
		}

		arrowAndButtons.dock = Dock.LEFT
		arrowAndButtons.setDockMargin(left = 4f)

		val arrowLine = EditablePanel(this, arrowAndButtons, y = 8f, height = 8f, width = arrowAndButtons.width)

		val autoRefillMatchMode = SmallSquareButtonPanel(this, arrowLine, skinElement = { REFILL_MATCH_MODE[menu.settings.ingredientMatchingMode.ordinal] }) {
			when (it) {
				0 -> menu.settings.ingredientMatchingMode = menu.settings.ingredientMatchingMode.next
				1 -> menu.settings.ingredientMatchingMode = menu.settings.ingredientMatchingMode.prev
			}

			menu.sendSettingsToServer()
		}

		val refillPriority = object : SmallSquareButtonPanel(this@ItemMonitorScreen, arrowLine, lambdaOnPress = {
			when (it) {
				0 -> menu.settings.ingredientPriority = menu.settings.ingredientPriority.next
				1 -> menu.settings.ingredientPriority = menu.settings.ingredientPriority.prev
			}

			menu.sendSettingsToServer()
		}) {
			override fun innerRender(stack: PoseStack, mouse_x: Float, mouse_y: Float, flag: Float) {
				super.innerRender(stack, mouse_x, mouse_y, flag)

				when (menu.settings.ingredientPriority) {
					ItemMonitorPlayerSettings.Target.SYSTEM -> ARROW.render(stack, 0f, 0f, width, height, UVWindingOrder.FLIP)
					ItemMonitorPlayerSettings.Target.INVENTORY -> ARROW.render(stack)
				}
			}
		}

		refillPriority.setDockMargin(2f)

		autoRefillMatchMode.dock = Dock.LEFT
		refillPriority.dock = Dock.LEFT

		val resultAndButtons = EditablePanel(this, bottomPanel, width = 18f)

		resultAndButtons.dock = Dock.LEFT
		resultAndButtons.setDockMargin(left = 6f)

		val craftingResultSlot = object : AbstractSlotPanel(this@ItemMonitorScreen, resultAndButtons, y = 18f) {
			override fun getItemStack(): ItemStack {
				return ItemStack(Items.BREAD, 666)
			}
		}

		object : SmallSquareButtonPanel(this@ItemMonitorScreen, resultAndButtons, y = 8f, lambdaOnPress = {
			when (it) {
				0 -> menu.settings.resultTarget = menu.settings.resultTarget.next
				1 -> menu.settings.resultTarget = menu.settings.resultTarget.prev
			}

			menu.sendSettingsToServer()
		}) {
			override fun innerRender(stack: PoseStack, mouse_x: Float, mouse_y: Float, flag: Float) {
				super.innerRender(stack, mouse_x, mouse_y, flag)

				when (menu.settings.resultTarget) {
					ItemMonitorPlayerSettings.Target.SYSTEM -> ARROW.render(stack, 0f, 0f, width, height, UVWindingOrder.FLIP)
					ItemMonitorPlayerSettings.Target.INVENTORY -> ARROW.render(stack)
				}
			}
		}

		SmallSquareButtonPanel(this, resultAndButtons, x = 10f, y = 8f, skinElement = { HOW_MUCH_TO_CRAFT[menu.settings.craftingAmount.ordinal] }) {
			when (it) {
				0 -> menu.settings.craftingAmount = menu.settings.craftingAmount.next
				1 -> menu.settings.craftingAmount = menu.settings.craftingAmount.prev
			}

			menu.sendSettingsToServer()
		}

		val craftingHistory = GridPanel(this, bottomPanel, width = 3 * 18f, height = 3 * 18f, columns = 3, rows = 3)
		craftingHistory.dock = Dock.LEFT
		craftingHistory.setDockMargin(left = 4f)

		val craftingHistoryScroll = ScrollBarPanel(this, bottomPanel, 0f, 0f, 0f)
		craftingHistoryScroll.dock = Dock.LEFT
		craftingHistoryScroll.setDockMargin(left = 2f)

		for (i in 0 until 9) {
			object : AbstractSlotPanel(this@ItemMonitorScreen, craftingHistory) {
				override fun getItemStack(): ItemStack {
					return ItemStack(Items.ARROW, 42)
				}

				override fun mouseScrolledInner(mouse_x: Double, mouse_y: Double, scroll: Double): Boolean {
					return craftingHistoryScroll.mouseScrolledInner(mouse_x, mouse_y, scroll)
				}
			}
		}

		return frame
	}

	companion object {
		const val FRAME_WIDTH = 240f
		const val FRAME_HEIGHT = 210f

		const val ITEM_GRID_WIDTH = 9
		const val ITEM_GRID_HEIGHT = 5

		val ARROW = SkinGrid.WIDGETS_8[1, 0]
		val S = SkinGrid.WIDGETS_8[2, 0]
		val F = SkinGrid.WIDGETS_8[3, 0]
		val E = SkinGrid.WIDGETS_8[4, 0]
		val ONE = SkinGrid.WIDGETS_8[5, 0]

		val HOW_MUCH_TO_CRAFT = listOf(ONE, S, F)
		val REFILL_MATCH_MODE = listOf(F, E)
	}
}
