package ru.dbotthepony.mc.otm.client.screen.panels

import com.mojang.blaze3d.vertex.PoseStack
import com.sun.jna.platform.win32.WinUser.SIZE
import net.minecraft.client.gui.components.Button
import net.minecraft.client.gui.components.Button.OnPress
import net.minecraft.client.resources.sounds.SimpleSoundInstance
import net.minecraft.network.chat.Component
import net.minecraft.sounds.SoundEvents
import ru.dbotthepony.mc.otm.client.minecraft
import ru.dbotthepony.mc.otm.client.render.RenderHelper
import ru.dbotthepony.mc.otm.client.render.SkinElement
import ru.dbotthepony.mc.otm.client.render.SkinGrid
import ru.dbotthepony.mc.otm.client.render.element
import ru.dbotthepony.mc.otm.client.screen.MatteryScreen

open class ButtonPanel(
	screen: MatteryScreen<*>,
	parent: EditablePanel?,
	x: Float = 0f,
	y: Float = 0f,
	width: Float = 40f,
	height: Float = 20f,
	label: Component
) : MinecraftWidgetPanel<Button>(screen, parent, x, y, width, height) {
	constructor(screen: MatteryScreen<*>, parent: EditablePanel?, label: Component) : this(screen, parent, x = 0f, label = label)

	var label = label
		set(value) {
			field = value
			widget?.message = value
		}

	override fun copyValues(new_widget: Button, old_widget: Button) {}
	override fun configureNew(widget: Button, recreation: Boolean) {}

	override fun makeNew(): Button {
		return object : Button(0, 0, width.toInt(), height.toInt().coerceAtMost(20), label, OnPress { this@ButtonPanel.onPress() }) {
			override fun isHoveredOrFocused(): Boolean {
				return this@ButtonPanel.isHovered
			}
		}
	}

	protected var callback: Runnable? = null

	fun bind(runnable: Runnable) {
		callback = runnable
	}

	protected open fun onPress() {
		callback?.run()
	}
}

@Suppress("PropertyName")
abstract class SquareButtonPanel(
	screen: MatteryScreen<*>,
	parent: EditablePanel?,
	x: Float = 0f,
	y: Float = 0f,
	width: Float,
	height: Float,
	val skinElement: (() -> SkinElement)? = null,
	val lambdaOnPress: ((clickButton: Int) -> Unit)? = null,
) : EditablePanel(screen, parent, x, y, width, height) {
	protected var pressed = false

	abstract val PRESSED: SkinElement
	abstract val HOVERED: SkinElement
	abstract val IDLE: SkinElement

	var isDisabled = false
		set(value) {
			if (field != value) {
				if (!value) {
					pressed = false
					trapMouseInput = false
				}

				field = value
			}
		}

	override fun innerRender(stack: PoseStack, mouse_x: Float, mouse_y: Float, flag: Float) {
		if (pressed) {
			PRESSED.render(stack, 0f, 0f, width, height)
		} else if (isHovered && !isDisabled) {
			HOVERED.render(stack, 0f, 0f, width, height)
		} else {
			IDLE.render(stack, 0f, 0f, width, height)
		}

		skinElement?.invoke()?.render(stack, 0f, 0f, width, height)
	}

	override fun mouseClickedInner(mouse_x: Double, mouse_y: Double, mouse_click_type: Int): Boolean {
		if (!isDisabled) {
			if (!pressed) {
				minecraft.soundManager.play(SimpleSoundInstance.forUI(SoundEvents.UI_BUTTON_CLICK, 1.0f))
			}

			pressed = true
			trapMouseInput = true
		}

		return true
	}

	override fun mouseReleasedInner(mouse_x: Double, mouse_y: Double, flag: Int): Boolean {
		if (!isDisabled && pressed) {
			pressed = false

			if (isHovered) {
				lambdaOnPress?.invoke(flag)
			}
		}

		trapMouseInput = false

		return true
	}
}

open class LargeSquareButtonPanel(
	screen: MatteryScreen<*>,
	parent: EditablePanel?,
	x: Float = 0f,
	y: Float = 0f,
	width: Float = SIZE,
	height: Float = SIZE,
	skinElement: (() -> SkinElement)? = null,
	lambdaOnPress: ((clickButton: Int) -> Unit)? = null,
) : SquareButtonPanel(screen, parent, x, y, width, height, skinElement, lambdaOnPress) {
	final override val IDLE = SkinGrid.WIDGETS_18[0, 0]
	final override val HOVERED = SkinGrid.WIDGETS_18[1, 0]
	final override val PRESSED = SkinGrid.WIDGETS_18[2, 0]

	companion object {
		const val SIZE = 18f
	}
}

open class SmallSquareButtonPanel(
	screen: MatteryScreen<*>,
	parent: EditablePanel?,
	x: Float = 0f,
	y: Float = 0f,
	width: Float = SIZE,
	height: Float = SIZE,
	skinElement: (() -> SkinElement)? = null,
	lambdaOnPress: ((clickButton: Int) -> Unit)? = null,
) : SquareButtonPanel(screen, parent, x, y, width, height, skinElement, lambdaOnPress) {
	final override val IDLE = SkinGrid.WIDGETS_8[0, 0]
	final override val HOVERED = SkinGrid.WIDGETS_8[0, 0]
	final override val PRESSED = SkinGrid.WIDGETS_8[0, 0]

	companion object {
		const val SIZE = 8f
	}
}
